@@ -397,7 +397,7 @@ from ae.core import DEBUG_LEVELS                                    # type: igno
 from ae.console import ConsoleApp                                   # type: ignore
 
 
-__version__ = '0.0.33'
+__version__ = '0.1.34'
 
 
 AppStateType = Dict[str, Any]                   #: app state config variable type
@@ -589,16 +589,16 @@ class MainAppBase(ConsoleApp, ABC):
         """
         return self.cfg_section_variable_names(APP_STATE_SECTION_NAME)
 
-    def change_app_state(self, state_name: str, new_value: Any, send_event: bool = True):
-        """ change single app state item to new_value in self.attribute and app_state dict item.
+    def change_app_state(self, state_name: str, state_value: Any, send_event: bool = True):
+        """ change app state to :paramref:`~change_app_state.state_value` in self.<state_name> and app_state dict item.
 
         :param state_name:      name of the app state to change.
-        :param new_value:       new value of the app state to change.
+        :param state_value:     new value of the app state to change.
         :param send_event:      pass False to prevent to send/call the on_<state_name> event to the main app instance.
         """
-        self.vpo(f"MainAppBase.change_app_state({state_name}, {new_value}, {send_event})"
+        self.vpo(f"MainAppBase.change_app_state({state_name}, {state_value}, {send_event})"
                  f" flow={self.flow_id} last={self._last_focus_flow_id} path={self.flow_path}")
-        self.change_observable(state_name, new_value, is_app_state=True)
+        self.change_observable(state_name, state_value, is_app_state=True)
         if send_event:
             self.call_method('on_' + state_name)
 
@@ -812,7 +812,7 @@ class MainAppBase(ConsoleApp, ABC):
                                 indexed widget the input focus (these exceptions are configurable via
                                 the list :data:`ACTIONS_CHANGING_FLOW_WITHOUT_CONFIRMATION`).
         """
-        self.dpo(f"MainAppBase.change_flow('{new_flow_id}', {popups_to_close}, {event_kwargs})"
+        self.vpo(f"MainAppBase.change_flow('{new_flow_id}', {popups_to_close}, {event_kwargs})"
                  f" flow='{self.flow_id}' path={self.flow_path}")
 
         action = flow_action(new_flow_id)
@@ -1029,22 +1029,41 @@ class MainAppBase(ConsoleApp, ABC):
         if self._stop_event_loop:
             self._stop_event_loop()         # will exit the self._start_event_loop() method called by self.run_app()
 
-    def widget_by_flow_id(self, flow_id: str) -> Optional[Any]:
-        """ determine the widget referenced by the passed flow_id.
+    def widget_by_attribute(self, att_name: str, att_value: str) -> Optional[Any]:
+        """ determine the widget referenced by the passed attribute name and value.
 
-        :param flow_id:         flow id referencing the focused widget.
-        :return:                widget that has the focus when the passed flow id is set.
+        :param att_name:        name of the attribute of the searched widget.
+        :param att_value:       attribute value of the searched widget.
+        :return:                widget that has the specified attribute with the specified value.
         """
         def child_wid(children):
             """ search children for widget.ae_flow_id attr value equal to :paramref:`~widget_by_flow_id.flow_id`. """
             for widget in children:
                 found = child_wid(widget.children)
-                if not found and getattr(widget, 'ae_flow_id', None) == flow_id:
+                if found:
+                    return found
+                elif getattr(widget, att_name, None) == att_value:
                     return widget
             return None
 
         return child_wid(self.framework_win.children)
 
+    def widget_by_flow_id(self, flow_id: str) -> Optional[Any]:
+        """ determine the widget having the passed flow_id.
+
+        :param flow_id:         flow id value of the searched widget's `ae_flow_id` attribute.
+        :return:                widget that has a `ae_flow_id` attribute with the value of the passed flow id.
+        """
+        return self.widget_by_attribute('ae_flow_id', flow_id)
+
+    def widget_by_state_name(self, state_name: str) -> Optional[Any]:
+        """ determine the widget having the passed app state name (ae_state_name).
+
+        :param state_name:      app state name of the widget's `ae_state_name` attribute.
+        :return:                widget that has a `ae_state_name` attribute with the passed app state name.
+        """
+        return self.widget_by_attribute('ae_state_name', state_name)
+
     def win_pos_size_change(self, *win_pos_size):
         """ screen resize handler (called on window resize or when app will exit/stop via closed event.
 
@@ -1054,3 +1073,7 @@ class MainAppBase(ConsoleApp, ABC):
         self.vpo(f"MainAppBase.win_pos_size_change landscape={self.framework_app.landscape}, {win_pos_size}")
         self.change_app_state('win_rectangle', win_pos_size)
         self.call_method('on_win_pos_size')
+
+
+module_globals = globals()
+#: used. e.g. by :mod:`ae.gui_help` for execution/evaluation of dynamic code, expressions and f-strings
